
/* 
 *######################## Demo PoStep60-256 ModbusRtu over RS485 communication #########################
 * Application demonstrates: 
 *                          -reading driver HW/FW info, voltage, mode, step mode
 *                          -driving a stepper motor with diferent speeds in both direction using commands: 
 *                           Run/Sleep, invert direction and set requested speed
 *  
 * Plese make sure you Enable Auto run mode and set Modbus configuration in PoStep60 applicatiion first! 
 * 
 *  
 * Modbus: 
 *        -function calls supported:
 *                                  -0x03 Read holding registers
 *                                  -0x06 Write single register
 *                                  -0x10 Write holding registers
 *        
 *        -Boudrate: 9600
 *        -Parity: Even
 *        -Stop bit: 1
 *        -Modbus address: 0x20
 *  
 *  
 * using Arduino MEGA2560 board: 
 *                                  Serial: Modbus
 *                                  Serial1: debug console (pin18->Tx)
 *                                  
 * Please read PoStep60 user manual for using other Modbus commands!                                   
 *######################################################################################################
*/


#include <ArduinoModbus.h>

#define HOLD_REG 0x02
#define SLAVE_ADDRESS 0x20;

int dir = 0;
int i,j = 0;
int data = 0;
int updown  = 0;
int change_dir  = 0;
int mbSlaveAddress = 0;
int TestEnd  = 0;
int SPEED_MAX = 0x0F; //pwm parameter

    

///////////////////////////////////////////////////
void setup() 
{
	mbSlaveAddress = SLAVE_ADDRESS;
	ModbusRTUClient.begin(9600, SERIAL_8E1);
	Serial1.begin(9600,SERIAL_8N1);

	// 0x0A – read HW/FW info
	PoStepModbusRead(0x0A, mbSlaveAddress);
	while(ModbusRTUClient.available())
	{
		Serial1.print("Slave ");
		Serial1.print(mbSlaveAddress, HEX);
		Serial1.print(": ");
		Serial1.println (ModbusRTUClient.read(), HEX);
	}
	Serial1.println("");
	delay(100);
          
	// 0x10 – read voltage
	PoStepModbusRead(0x10, mbSlaveAddress);
	while(ModbusRTUClient.available())
	{
		Serial1.print("Slave ");
		Serial1.print(mbSlaveAddress, HEX);
		Serial1.print(": ");
		Serial1.println ((ModbusRTUClient.read())*0.072, 2);
	}
	Serial1.println("");
	delay(100);
	
	// 0x14 – read driver mode
	PoStepModbusRead(0x14, mbSlaveAddress);
	while(ModbusRTUClient.available())
	{
		Serial1.print("Slave ");
		Serial1.print(mbSlaveAddress, HEX);
		Serial1.print(": ");
		Serial1.println (ModbusRTUClient.read(), HEX);
	}
	delay(100);
	Serial1.println("");

	// 0x23 – read step mode
	PoStepModbusRead(0x23, mbSlaveAddress);
	while(ModbusRTUClient.available())
	{
		Serial1.print("Slave ");
		Serial1.print(mbSlaveAddress, HEX);
		Serial1.print(": ");
		Serial1.println (ModbusRTUClient.read(), HEX);
	}
	delay(100);
	Serial1.println("");

	// 0x03 Run/sleep mode
	PoStepModbusWrite(0x03,mbSlaveAddress, 0xDA, 0, 0);
	delay(200);
	
	// 0x55 Set invert direction
	PoStepModbusWrite(0x55, mbSlaveAddress, (dir & 0x01), 0,0);  
}

void loop() 
{
	if(updown == 0)
	{
		if(data <= SPEED_MAX)
			data = data + 1;
		else
			updown = 1;
	}
	else
	{
		if(data >= 1)
			data = data - 1;
		else
		{
			updown = 0;
			change_dir =1;
			dir =~dir;
		}
	}

	delay(200); 

	if (TestEnd != 1)
	{
		if(change_dir == 1) 
		{
			change_dir =0;
			PoStepModbusWrite(0x55, mbSlaveAddress, (dir & 0x01), 0,0); 
			j++; 
			delay(200);   
		}

		if (j>1)
		{
			PoStepModbusWrite(0x0003, mbSlaveAddress, 0x000F, 0, 0);
			j = 0;
			TestEnd =1; 
			Serial1.println("Test done!");
		}
		else
			PoStepModbusWrite(0x54, mbSlaveAddress, data<<8, 0,0);
	}         
}


void PoStepModbusRead(int Rd_command, int slaveID)
{
	switch (Rd_command)
	{
		case 0x0A:
		ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x0A, 0x03);
		Serial1.println ("# 0x0A->Read HW/FW info:");  
		break;
    
		case 0x10:
		ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x10, 0x01);
		Serial1.println ("# 0x10->Read voltage:");  
		break;
    
    case 0x11:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x11, 0x01);
       Serial1.println ("# 0x11->Read temperature:");  
    break;
    
    case 0x12:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x12, 0x01);
       Serial1.println ("# 0x12->Read pin status:");  
    break;
    
    case 0x13:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x13, 0x01);
       Serial1.println ("# 0x13->Read driver status:");  
    break;
    
    case 0x14:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x14, 0x01);
       Serial1.println ("# 0x14->Read driver mode:");  
    break;
    
    case 0x20:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x20, 0x01);
       Serial1.println ("# 0x20->Read full scale current:");  
    break;
    
    case 0x21:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x21, 0x01);
       Serial1.println ("# 0x21->Read idle current:");  
    break;
    
    case 0x22:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x22, 0x01);
       Serial1.println ("# 0x22->Read overheat curent:");  
    break;
    
    case 0x23:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x23, 0x01);
       Serial1.println ("# 0x23->Read step mode:");  
    break;
    
    case 0x24:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x24, 0x01);
       Serial1.println ("# 0x24->Read temperature limit:");  
    break;
    
    case 0x25:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x25, 0x01);
       Serial1.println ("# 0x25->Read faults:");  
    break;        
    
    case 0x40:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x40, 0x02);
       Serial1.println ("# 0x40->Read position:");  
    break;
    
    case 0x41:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x41, 0x01);
       Serial1.println ("# 0x41->Read maximal speed:");  
    break;
    
    case 0x42:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x42, 0x01);
       Serial1.println ("# 0x42->Read acceleration:");  
    break;
    
    case 0x43:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x43, 0x01);
       Serial1.println ("# 0x43->Read deceleratio:");  
    break;
    
    case 0x44:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x44, 0x01);
       Serial1.println ("# 0x44->Read current speed:");  
    break;
    
    case 0x45:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x45, 0x01);
       Serial1.println ("# 0x45->Read requested speed:");  
    break;
    
    case 0x46:
       ModbusRTUClient.requestFrom(slaveID, HOLD_REG, 0x46, 0x01);
       Serial1.println ("# 0x46->Read Auto run invert direction status: ");  
    break;
                            
    default:
       Serial1.println ("This read command doesn't exist!");  
    break;
  }
}


void PoStepModbusWrite(int Wr_command, int slaveID, int command1, int command2, int command3)
{
  switch (Wr_command)
  {
    case 0x03:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x03,command1);
      Serial1.print ("$ 0x03->Set Run/Sleep mode: "); 
      Serial1.println (command1, HEX);       
    break;

    case 0x04:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x04,command1);
      Serial1.print ("$ 0x04->Set Modbus address: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x05:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x05,command1);
      Serial1.print ("$ 0x05->Set driver mode: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x06:
      ModbusRTUClient.beginTransmission(slaveID,HOLDING_REGISTERS,0x06,3);
      ModbusRTUClient.write(command1);
      ModbusRTUClient.write(command2);
      ModbusRTUClient.write(command3);
      ModbusRTUClient.endTransmission();
      Serial1.print ("$ 0x06->Set PWM in DC motor mode: ");
      Serial1.print (command1, HEX);
      Serial1.print (" ");  
      Serial1.print (command2, HEX); 
      Serial1.print (" "); 
      Serial1.println (command3, HEX); 
    break;
    
    case 0x30:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x30,command1);
      Serial1.print ("$ 0x30->Set ful scale current: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x31:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x31,command1);
      Serial1.print ("$ 0x31->Set idle current: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x32:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x32,command1);
      Serial1.print ("$ 0x32->Set overheat current: "); 
      Serial1.println (command1, HEX);  
    break;
    
    case 0x33:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x33,command1);
      Serial1.print ("$ 0x33->Set step mode: "); 
      Serial1.println (command1, HEX);  
    break;
    
    case 0x34:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x34,command1);
      Serial1.print ("$ 0x34->Set temperature limit: "); 
      Serial1.println (command1, HEX);  
    break;
    
    case 0x35:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x35,command1);
      Serial1.println ("$ 0x35 -Reset faults!"); 
    break;
    
    case 0x3F:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x3F,command1);
      Serial1.println ("$ 0x3F -Write settings to EEPROM "); 
    break;
    
    case 0x50:
      ModbusRTUClient.beginTransmission(slaveID,HOLDING_REGISTERS,0x50,2);
      ModbusRTUClient.write(command1);
      ModbusRTUClient.write(command2);
      ModbusRTUClient.endTransmission();
      Serial1.print ("$ 0x50->Set position: ");
      Serial1.print (command1, HEX);
      Serial1.println (command2, HEX); 
    break;        
    
    case 0x51:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x51,command1);
      Serial1.print ("$ 0x51->Set maximal speed: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x52:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x52,command1);
      Serial1.print ("$ 0x52->Set acceleration: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x53:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x53,command1);
      Serial1.print ("$ 0x53->Set deceleration: "); 
      Serial1.println (command1, HEX);  
    break;
    
    case 0x54:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x54,command1);
      Serial1.print ("$ 0x54->Set requested speed: "); 
      Serial1.println (command1, HEX);
    break;
    
    case 0x55:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x55,command1);
      Serial1.print ("$ 0x55->Set invert direction: "); 
      Serial1.println (command1, HEX); 
    break;
    
    case 0x5E:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x5E,command1);
      Serial1.println ("$ 0x5E->Set zero!");  
    break;
    
    case 0x5F:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x5F,command1);
      Serial1.println ("$ 0x5F -Stop!");  
    break;
    
    case 0x60:
      ModbusRTUClient.holdingRegisterWrite(slaveID,0x60,command1);
      Serial1.println ("$ 0x60 -System reset!");  
    break;
                    
    default:
       Serial1.println ("This write command doesn't exist!");  
    break;
  }
}
